#!/usr/bin/env python

from __future__ import absolute_import, division, print_function

from os import path
import os

from platform import system
from setuptools import Command, find_packages, setup
from setuptools.command.install import install

import sys

from slim import __version__

description = 'Splunk Packaging Toolkit'
name = 'splunk-packaging-toolkit'


class CreateSymlink(Command):
    def __init__(self, dist, **kwargs):
        Command.__init__(self, dist, **kwargs)
        self.install_scripts = None
        self.outputs = []

    def finalize_options(self):
        pass

    def get_outputs(self):
        return self.outputs

    def initialize_options(self):
        pass

    def run(self):
        executable = path.join(self.install_scripts, 'slim')
        link = '/usr/local/bin/slim'

        def create_symlink():
            if path.islink(link):
                os.remove(link)
            os.symlink(executable, link)

        self.execute(create_symlink, (), 'linking ' + link + ' -> ' + executable)
        self.outputs.append(link)


class Install(install):

    def run(self):
        if self.should_create_symlink():
            self.distribution.command_options['create_symlink'] = {
                'install_scripts': ('install command', self.install_scripts)
            }
        install.run(self)

    # We should create a symlink to the Python executable if we're running on Darwin or Linux and we're
    #
    # * executing in the system Python runtime on a Darwin or Linux system and
    #   In a virtual environment `sys.prefix` points to the virtualenv directory while `sys.real_prefix` points to the
    #   prefix of the system Python. In a non-virtual environment `sys.real_prefix` is undefined.
    #
    # * installing to the system site-packages directory
    #   We know that we are installing to the system site-packages directory if `self.install_base` is equal to
    #   `self.install_platbase`.

    _should_create_symlink = None

    def should_create_symlink(self):
        value = self._should_create_symlink
        if value is None:
            value = self._should_create_symlink = (
                not hasattr(sys, 'real_prefix') and
                system() in ('Darwin', 'Linux') and
                self.install_base == self.install_platbase
            )
        return value

    new_commands = [('create_symlink', should_create_symlink)]

    sub_commands = (
        [cmd for cmd in install.sub_commands if cmd[0] not in new_commands] + new_commands
    )


if sys.version_info < (2, 7):
    raise NotImplementedError('The ' + description + ' requires Python 2.7')

setup(
    name=name,
    version=__version__,
    description=description,
    url='https://dev.splunk.com',
    author='Splunk, Inc.',
    author_email='devinfo@splunk.com',
    license='https://www.splunk.com/en_us/legal/splunk-pre-release-software-license-agreement.html',
    classifiers=[
        'Programming Language :: Python',
        'Development Status :: 3 - Alpha',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'License :: Other/Proprietary License :: Splunk Pre-release Software License',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ],

    entry_points={'console_scripts': ['slim = slim.__main__:main']},
    packages=find_packages(exclude=['contrib', 'docs', 'test', '.pep8']),
    package_data={
        'slim': [
            'config/conf-specs/*.spec',
            'config/common-information-models.json',
            'config/splunk-releases.json',
            'config/settings'
        ]
    },
    install_requires=['semantic_version>=2.5.0'],

    cmdclass={'install': Install, 'create_symlink': CreateSymlink}
)
