#!/usr/bin/env python
# coding=utf-8
#
# Copyright © Splunk, Inc. All Rights Reserved.

from __future__ import absolute_import, division, print_function, unicode_literals

import sys

from slim.utils import *
from slim.app import *
from slim.command import *


# Argument parser definition

parser = SlimArgumentParser(description='''
    Generate a JSON manifest file for a Splunk app based on app configuration settings.''')

parser.add_app_directory()
parser.add_output_file(description='app manifest')


# pylint: disable=redefined-outer-name
def generate_manifest(source, output, add_defaults=True):

    # Run parse-conf to get the app configuration JSON
    SlimLogger.step('Parsing app configuration at ', encode_filename(source), '...')
    app_configuration = AppConfiguration.load(source)
    SlimLogger.exit_on_error()

    # Generate the manifest
    SlimLogger.step('Generating app manifest to ', encode_filename(output.name), '...')
    AppManifest.generate(app_configuration, output, add_defaults)
    SlimLogger.exit_on_error()

    # Prompt the user to update dependencies and forwarderGroups
    SlimLogger.information('App manifest saved to ', encode_filename(output.name))


def main(argv):

    arguments = parser.parse_args(argv[1:])
    generate_manifest(arguments.source, arguments.output)


if __name__ == '__main__':
    # noinspection PyBroadException
    try:
        main(sys.argv)
    except SystemExit:
        raise
    except:
        SlimLogger.fatal(exception_info=sys.exc_info())
