#!/usr/bin/env python
# coding=utf-8
#
# Copyright © Splunk, Inc. All Rights Reserved.

from __future__ import absolute_import, division, print_function, unicode_literals

import io
import errno
import os
import shutil

# urllib2 is undefined in the Python 3.x library
# noinspection PyCompatibility
import urllib2

download_server = 'https://repo.splunk.com/artifactory/simple/test_dev_data/slim'


def print_verbose(msg):
    print('+ ' + msg)


def sha1_file(path):
    import hashlib

    try:
        sha_ctx = hashlib.sha1()
    except ImportError:
        import sha
        sha_ctx = sha.new()

    with io.open(path, 'rb') as f:
        while True:
          chunk = f.read(1024)
          if not chunk:
              break
          sha_ctx.update(chunk)

    return sha_ctx.hexdigest()


# cross platform and avoid race condition
def make_sure_path_exists(path):
    try:
        os.makedirs(path)
    except OSError as exception:
        if exception.errno != errno.EEXIST:
            raise


def download_test_data(name_version, expected_sha1, full_dir):

    if not expected_sha1:
        print('There is no SHA1 for "{0}", please add one to the "build()" invocation.'.format(name_version))
        return 1
        
    print_verbose('{{builtin}} Attempt download for "{0}"'.format(name_version))
    full_url = '{0}/{1}'.format(download_server, name_version)

    try:
        full_path = os.path.join(full_dir, name_version)
        if (not os.path.isfile(full_path)) or (not sha1_file(full_path) == expected_sha1):
            make_sure_path_exists(full_dir)
            download_stream = urllib2.urlopen(full_url)
            if os.path.isfile(full_path):
                os.remove(full_path)
            local_file = open(full_path, 'wb')
            shutil.copyfileobj(download_stream, local_file)
            download_stream.close()
            local_file.close()
            print_verbose('{{builtin}} Downloaded {0} to {1}'.format(full_url, full_path))
        else:
            print('file is already there, no need to download')
        
        if expected_sha1 and not sha1_file(full_path) == expected_sha1:
            os.remove(full_path)
            print('The SHA1 of "{0}" did not match the expected SHA1 (expected "{1}")'.format(full_path, expected_sha1))
            return 1

        return 0
    
    except urllib2.HTTPError as error:
        print('failed downloading file from "{0}": {1}'.format(full_url, error))
        return 1
