#!/usr/bin/env python
# coding=utf-8
#
# Copyright © Splunk, Inc. All Rights Reserved.

from __future__ import absolute_import, division, print_function, unicode_literals

import io
import os
import sys
import shutil
import argparse

# urllib2 is undefined in the Python 3.x library
# noinspection PyCompatibility
import urllib2


BASE_URL = "https://repo.splunk.com/artifactory/simple/test_dev_data/slim"

parser = argparse.ArgumentParser(fromfile_prefix_chars='@')
parser.add_argument('--archive_name', required=True)
parser.add_argument('--expected_sha1', required=True)
parser.add_argument('--destination', required=True)
args = parser.parse_args(sys.argv[1:])

archive_name = args.archive_name    # test_generate_manifest-empty_app-1.0.0.tar.gz
expected_sha1 = args.expected_sha1  # 9f8fb5f9174d2adff35e7c151df2077962fe8f5a
destination = args.destination      # empty_app

full_url = "%s/%s-%s" % (BASE_URL, archive_name, expected_sha1)
full_path = os.path.join(destination, archive_name)

print("Current directory => %s" % (os.getcwd()))

print("Attempting to download %s to %s" %(full_url, full_path))

def sha1_file(path):
    import hashlib

    try:
        sha_ctx = hashlib.sha1()
    except ImportError:
        import sha
        sha_ctx = sha.new()

    with io.open(path, 'rb') as f:
        while True:
            chunk = f.read(1024)
            if not chunk:
                break
            sha_ctx.update(chunk)

    return sha_ctx.hexdigest()

try:

    # Download the archive to tmp file location
    full_path_tmp = full_path + '.download'
    local_file = open(full_path_tmp, 'wb')

    download_stream = urllib2.urlopen(full_url)
    shutil.copyfileobj(download_stream, local_file)

    download_stream.close()
    local_file.close()

except urllib2.HTTPError, e:
    print("No remote download was found for '%s': %s -- %s" % (full_url, e.code, e.msg))
    sys.exit(1)

# Check the sha1 hash of the downloaded archive
if expected_sha1 and not sha1_file(full_path_tmp) == expected_sha1:
    os.remove(full_path_tmp)
    print("The SHA1 of '%s' did not match the expected SHA1 (expected '%s')" % (full_path, expected_sha1))

# Rename the downloaded archive to the final destination
os.rename(full_path_tmp, full_path)

print("Downloaded %s to %s" % (full_url, full_path))
