#!/usr/bin/env python
# coding=utf-8
#
# Copyright © Splunk, Inc. All Rights Reserved.

from __future__ import absolute_import, division, print_function, unicode_literals

from os import environ, mkdir, path
from shutil import copyfile, rmtree
from tempfile import mkdtemp

import io

from slim.app import AppConfigurationSpec
from slim.utils import slim_configuration, splunk
from slim.utils.internal import string

from unittest import TestCase


class TestAppConfigurationSpec(TestCase):

    # region Tests

    def test_load(self):
        configuration = 'inputs'

        master_filename = path.join(slim_configuration.configuration_spec_path, configuration + '.conf.spec')
        spec = AppConfigurationSpec.load(master_filename)
        replica_filename = path.join(self._test_directory, configuration + '.conf.spec')
        spec.save(replica_filename)

        with io.open(master_filename) as f:
            master = f.readlines()

        with io.open(replica_filename) as f:
            replica = f.readlines()

        try:
            self.assertEqual(master, replica)
        except AssertionError as error:

            if not path.isdir(self._save_directory):
                mkdir(self._save_directory)

            target = path.join(self._save_directory, configuration + '.conf.spec-master')
            copyfile(master_filename, target)
            master_filename = target

            target = path.join(self._save_directory, configuration + '.conf.spec-replica')
            copyfile(replica_filename, target)
            replica_filename = target

            self.fail(string(error) + '\ndiff ' + master_filename + ' ' + replica_filename)

    # endregion

    # region Setup and teardown

    @classmethod
    def setUpClass(cls):
        cls._test_directory = mkdtemp(prefix='slim-' + path.splitext(path.basename(__file__))[0] + '.', dir=environ['SPLUNK_SOURCE'])

    @classmethod
    def tearDownClass(cls):
        rmtree(cls._test_directory)

    # endregion

    # region Privates

    _test_directory = None
    _system_readme_directory = path.join(splunk.etc, 'system', 'README')
    _save_directory = path.splitext(_test_directory)[0]

    # endregion
    pass


if __name__ == '__main__':
    import unittest.main
    unittest.main()
