#!/usr/bin/env python
# coding=utf-8
#
# Copyright © Splunk, Inc. All Rights Reserved.

from __future__ import absolute_import, division, print_function, unicode_literals
from unittest import TestCase

from io import StringIO
import io

import sys
import semantic_version

from slim.app._internal import ObjectView
from slim.app import AppManifest


class TestAppManifest(TestCase):

    count, path = 0, None

    @classmethod
    def setUpClass(cls):

        print('\nVariant ', cls.count, ': ', cls.path, file=sys.stderr, sep='')

        with io.open(cls.path, encoding='utf-8') as istream:
            cls.app_manifest = AppManifest.load(istream)

    def test_describe(self):

        with StringIO(newline=None) as ostream:
            self.app_manifest.describe(ostream)
            observed = ostream.getvalue()

        with io.open(self.path + '.describe.expected', encoding='utf-8') as istream:
            expected = istream.read()

        self.assertEqual(observed, expected)

    def test_generate(self):
        pass

    def test_load(self):
        app_manifest = self.app_manifest

        expected_elements = frozenset((
            'schemaVersion',
            'info',
            'dependencies',
            'tasks',
            'inputGroups',
            'platformRequirements',
            'incompatibleApps'
        ))

        self.assertEqual(frozenset((name for name in app_manifest)), expected_elements)

        self.assertEqual(app_manifest.schemaVersion, semantic_version.Version.coerce('1.0'))
        self.assertIsInstance(app_manifest.info, ObjectView)
        self.assertIsInstance(app_manifest.dependencies, (type(None), ObjectView))
        self.assertIsInstance(app_manifest.tasks, (type(None), list))
        self.assertIsInstance(app_manifest.inputGroups, (type(None), ObjectView))
        self.assertIsInstance(app_manifest.platformRequirements, (type(None), ObjectView))
        self.assertIsInstance(app_manifest.incompatibleApps, (type(None), ObjectView))

        return

    def test_save(self):
        app_manifest = self.app_manifest


if __name__ == '__main__':
    # noinspection PyUnresolvedReferences
    from common import test_variants
    test_variants(TestAppManifest, file=__file__, pattern='*.manifest')
