#!/usr/bin/env python
# coding=utf-8
#
# Copyright © 2015 Splunk, Inc. All Rights Reserved.

from __future__ import absolute_import, division, print_function, unicode_literals

from os import urandom
from random import randint
from struct import unpack_from

import unicodedata

from slim.utils.internal import char

__all__ = ['generate_random_string']


def generate_random_string(minimum, maximum):

    n = randint(minimum, maximum)
    result = ''

    while n > 0:
        buffer = urandom(2 * n)
        for offset in range(0, len(buffer), 2):
            i = unpack_from('H', buffer, offset)[0]
            try:
                code_point = char(i)
            except ValueError:
                continue
            if unicodedata.category(code_point) not in _printable_character_categories:
                continue
            result += code_point
            n -= 1

    return result

_printable_character_categories = frozenset((  # from http://www.fileformat.info/info/unicode/category/index.htm
    'LC',  # Letter, Cased
    'Ll',  # Letter, Lowercase
    'Lm',  # Letter, Modifier
    'Lo',  # Letter, Other
    'Lt',  # Letter, Titlecase
    'Lu',  # Letter, Uppercase
    'Mc',  # Mark, Spacing Combining
    'Me',  # Mark, Enclosing
    'Mn',  # Mark, Nonspacing
    'Nd',  # Number, Decimal Digit
    'Nl',  # Number, Letter
    'No',  # Number, Other
    'Pc',  # Punctuation, Connector
    'Pd',  # Punctuation, Dash
    'Pe',  # Punctuation, Close
    'Pf',  # Punctuation, Final quote (may behave like Ps or Pe depending on usage)
    'Pi',  # Punctuation, Initial quote (may behave like Ps or Pe depending on usage)
    'Po',  # Punctuation, Other
    'Ps',  # Punctuation, Open
    'Sc',  # Symbol, Currency
    'Sk',  # Symbol, Modifier
    'Sm',  # Symbol, Math
    'So',  # Symbol, Other
    'Zl',  # Separator, Line
    'Zp',  # Separator, Paragraph
    'Zs',  # Separator, Space
    ))
