#!/usr/bin/env python
# coding=utf-8
#
# Copyright © Splunk, Inc. All Rights Reserved.

from __future__ import absolute_import, division, print_function, unicode_literals

from slim.utils import SlimLogger, SlimLoggerLevel, SlimPayload, SlimPayloadLoggingHandler
from unittest import TestCase
import sys


class TestSlimLogger(TestCase):

    # region Tests

    def test_configure(self):
        pass

    def test_log(self):
        pass

    def test_payload_logging_handler(self):

        payload_logging_handler = SlimPayloadLoggingHandler()
        SlimLogger.add_handler(payload_logging_handler)

        SlimLogger.debug('1.', ' debug ', ' message')
        SlimLogger.error('2.', ' error', ' message')
        SlimLogger.information('4. ', ' information', ' message')
        SlimLogger.step('5.', ' step', ' message')
        SlimLogger.step('6.', ' step', ' message')

        try:
            SlimLogger.fatal('3.', ' fatal', ' message')
        except BaseException as exception:
            self.assertIsInstance(exception, SystemExit)
            self.assertEqual(exception.code, 1)
        else:
            self.fail('Expected SystemExit, not a return from SlimLogger.fatal')

        try:
            raise ValueError('test')
        except BaseException as exception:
            try:
                SlimLogger.fatal('3.', ' fatal', ' message', exception_info=sys.exc_info())
            except BaseException as exception:
                self.assertIsInstance(exception, SystemExit)
                self.assertEqual(exception.code, 1)
            else:
                self.fail('Expected SystemExit, not a return from SlimLogger.fatal')

        for level_no, level_name in enumerate(SlimLoggerLevel):
            try:
                SlimLogger.message(level_no, 'level_no, level_name = ', level_no, ', ', SlimLoggerLevel[level_no])
            except BaseException as exception:
                self.assertIsInstance(exception, SystemExit)
                self.assertEqual(exception.code, 1)
                self.assertEqual(SlimLoggerLevel.FATAL, level_no)
            else:
                if level_no == SlimLoggerLevel.FATAL:
                    self.fail('Expected SystemExit, not a return from SlimLogger.message for a fatal error')
                pass

        payload = SlimPayload()
        payload_logging_handler.payload = payload
        self.assertIs(payload_logging_handler.payload, payload)

        another_payload_logging_handler = SlimPayloadLoggingHandler(payload)
        self.assertIs(another_payload_logging_handler.payload, payload)

    # endregion

    # region Setup and teardown

    def setUp(self):
        # SlimLogger.reset()
        pass

    # endregion
    pass


if __name__ == '__main__':
    import unittest.main
    unittest.main()
