#!/usr/bin/env python
# coding=utf-8
#
# Copyright © Splunk, Inc. All Rights Reserved.

from __future__ import absolute_import, division, print_function, unicode_literals

import io
import os
import sys

from os import path

test_dir = path.dirname(path.join(path.abspath(__file__)))
project_dir = path.dirname(test_dir)
sys.path.insert(0, project_dir)

from slim.app import AppManifest
from slim.utils import SlimLogger

manifest_dir = path.join(test_dir, 'manifests')
print('SLIM: test_validate_manifest')
status = 0

for app_manifest_file in sorted(os.listdir(manifest_dir)):

    if not app_manifest_file.startswith('app.manifest.'):
        continue  # Unexpected file (e.g., .DS_Store on OS X)

    app_manifest = path.join(manifest_dir, app_manifest_file)

    with io.open(app_manifest, encoding='utf-8') as istream:

        # Load the manifest file (errors are logged)
        AppManifest.load(istream)

        # Determine expected/observed results (based on logged errors)
        expected_errors = ('FAIL' in app_manifest_file)
        observed_errors = (SlimLogger.error_count() > 0)

        expected_message = 'expected manifest errors' if expected_errors else 'expected no manifest errors'

        # Report the test result
        if expected_errors != observed_errors:
            print('[TEST_FAIL] ' + app_manifest_file + ': ' + expected_message)
            status = 1
        else:
            print('[TEST_PASS] ' + app_manifest_file)

        SlimLogger.reset_counts()

sys.exit(status)
