#!/usr/bin/python

import urllib2
import sys
import json
import getpass
import base64

BASE_URL = "https://repo.splunk.com/artifactory/simple/test_dev_data/slim"

if len(sys.argv) < 2:
    print("Upload requires at least one archive name.")
    sys.exit(1)

archives = sys.argv[1:]

USERNAME = raw_input("Please enter your AD username: ")
PASSWORD = getpass.getpass("Please enter your AD password: ")

for archive in archives:
    # We read the entire file contents into memory, but mostly because I'm not sure
    # how to get a file upload working with urllib2 without doing this.
    archive_file = open(archive, 'rb')
    archive_contents = archive_file.read()
    archive_file.close()

    sha_ctx = None
    try:
        import hashlib
        sha_ctx = hashlib.sha1()
    except ImportError:
        import sha
        sha_ctx = sha.new()
    sha_ctx.update(archive_contents)
    sha1 = sha_ctx.hexdigest()
    
    url = "%s/%s-%s" % (BASE_URL, archive, sha1)
    
    base64string = base64.encodestring('%s:%s' % (USERNAME, PASSWORD)).replace('\n', '')
    req = urllib2.Request(url, archive_contents, {'Content-Type': 'application/octet-stream'})
    req.add_header("Authorization", "Basic %s" % base64string)   
    req.get_method = lambda: 'PUT'
    
    try:
        r = urllib2.urlopen(req)
    except urllib2.HTTPError, e:
        if e.code == 401:
            print "Incorrect username/password, or you may not be allowed to upload to contrib"
        else:
            print "Received error: %s -- %s" % (e.code, e.msg)
        sys.exit(1)
        
    code = r.getcode()
    response = r.read()
    parsed_response = json.loads(response)

    if code != 201:
        print "There was an error uploading to the artifactory server (code %s), here is the response: \n%s" % (code, response)
        
    print "UPLOADED: %s" % archive
    print "  URL: %s" % parsed_response["downloadUri"]
    print "  SHA1: %s" % parsed_response["checksums"]["sha1"]
